package cz.cvut.agents.rph.core;

import cz.cvut.agents.rph.reversi.ReversiGameResult;

import java.util.ArrayList;
import java.util.Collection;

/**
 * Model of the game - should store all the data about the game, no GUI no logic here.
 * Author: Ondrej Vanek
 * Date: 10/3/12
 * Time: 9:51 AM
 */
public abstract class GameModel {

    /**
     * Collection of views.
     */
    private Collection<GameView> views = new ArrayList<GameView>();

    /**
     * Actual status of the game.
     */
    protected GameStatus gameStatus = GameStatus.PAUSED;


    public final void addView(GameView view){
        views.add(view);
    }

    public final void removeView(GameView view){
        views.remove(view);
    }

    public final void notifyViews(){
        for(GameView view:views){
            view.update();
        }
    }

    public abstract void restart();

    /**
     * TODO: The notification of the views is done here - not decided yet, if it's clever.
     * @param gameStatus
     */
    public void setGameStatus(GameStatus gameStatus) {
        this.gameStatus = gameStatus;
        notifyViews();
    }

    public abstract ReversiGameResult getGameResult();

    public abstract boolean isEndOfGame();

    public GameStatus getGameStatus() {
        return gameStatus;
    }


    /**
     * Status of the game.
     */
    public static  enum GameStatus {PAUSED,PLAYING,FINISHED;

        public static String getButtonText(GameStatus status){
            switch (status){
                case PAUSED: return "Play";
                case PLAYING: return "Pause";
                case FINISHED : return "Restart";
                default: throw new IllegalArgumentException("Unknown game status:"+status);
            }
        }
    }
}
