package cz.cvut.agents.rph.core;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ArrayList;
import java.util.List;

/**
 * Represents a view of a game.
 * Author: Ondrej Vanek
 * Date: 10/3/12
 * Time: 9:51 AM
 */
public abstract class GameView {

    /**
     * List of listeners listening to the action events from the view.
     */
    private List<ActionListener> listeners = new ArrayList<ActionListener>();

    /**
     * Adds the action listener to listen for the events of the view.
     * @param listener
     */
    public void addActionListener(ActionListener listener){
        listeners.add(listener);
    }

    /**
     * Removes an action listener.
     * @param listener
     */
    public void removeActionListener(ActionListener listener){
        listeners.remove(listener);
    }

    /**
     * Notifies listeners about the event e.
     * @param e
     */
    public void notifyListeners(ActionEvent e){
        for(ActionListener listener : listeners){
            listener.actionPerformed(e);
        }
    }

    /**
     * Updates view based on the call from the {@link GameModel} - the model was changed.
     */
    public abstract void update();

}
