package cz.cvut.agents.rph.reversi;

import java.util.Comparator;


/**
 * Data Structure representing Reversi result of a player.
 * @author Ondrej Vanek
 *
 */
public class PlayerResult implements Comparable<PlayerResult> {
	
    private String playerName; 
    private int score;
    private int stonesWon;
    private ReversiScoreEvaluator.ResultType resultType;


    private static Comparator<PlayerResult> comparator = null;

    public PlayerResult(String playerName, int score, int stonesWon, ReversiScoreEvaluator.ResultType resultType) {
    	this.playerName = playerName;
        this.score = score;

        //assign stones only if there was no error and no timeout
        if(!resultType.equals(ReversiScoreEvaluator.ResultType.ERROR) && !resultType.equals(ReversiScoreEvaluator.ResultType.TIMEOUT)){
            this.stonesWon = stonesWon;
        }
        this.resultType = resultType;
    }

    public String getPlayerName() {
        return playerName;
    }

    public int getScore() {
        return score;
    }

    public int getStonesWon() {
        return stonesWon;
    }

    public ReversiScoreEvaluator.ResultType getResultType() {
        return resultType;
    }

    /**
     * Adds score and stone to the current result.
     * @param score
     * @param stonesWon
     */
    public void recordGameResults(int score, int stonesWon) {
        this.score += score;
        this.stonesWon += stonesWon;
    }

    public String toString() {
        return playerName +" " + resultType +" > Score:" + score + ", Stones:" + stonesWon;
    }

    public int compareTo(PlayerResult p2) {
        int c1 = p2.score - this.score;
        if (c1 == 0)
            return p2.stonesWon - this.stonesWon;
        else
            return c1;       
    }

    public String getCSVLine(){
        return playerName +", " + resultType +", " + score + ", " + stonesWon;
    }

    public static String getCSVHeader(){
        return "player, result, score, stones";
    }
    
    /**
     * @return Comparator comparing player results.
     */
    public static Comparator<PlayerResult> getComparator(){
    	if(comparator==null) {
    		comparator = new PlayerResultComparator();
    	}
    	return comparator;
    }
    
    /**
     * Compares {@link PlayerResult}s according to games won, then according to stones gathered.
     * @author Ondrej Vanek
     *
     */
    private static class PlayerResultComparator implements Comparator<PlayerResult> {
       
        public int compare(PlayerResult player1, PlayerResult player2) {
            int result = player1.score - player2.score;
            if (result == 0)
                return player1.stonesWon - player2.stonesWon;
            else
                return result;
        }


}
}
