package cz.cvut.agents.rph.reversi;

import javax.swing.*;
import java.awt.*;

/**
 * Represents panel of the board.
 * Author: Premysl Volf
 * Date: 10/3/12
 * Time: 10:09 AM
 */
public class PlaygroundPanel extends JPanel {

	private static final long serialVersionUID = 8315771387404619901L;
	static final Color[] COLOR_POOL = new Color[]{
        new Color(255,  96,  96),
        new Color( 96,  96, 255),
        new Color(255,  32,  32),
        new Color( 32,  32, 255),
    	new Color(255,  32, 128),
    	new Color(128,  32, 255)};

	private ReversiModel model;

    public PlaygroundPanel(ReversiModel model) {
        this.model = model;
    }

    @Override
	protected void paintComponent(Graphics g) {
		super.paintComponent(g);
        Graphics2D g2d = (Graphics2D) g;
        g2d.setRenderingHint(RenderingHints.KEY_TEXT_ANTIALIASING,RenderingHints.VALUE_TEXT_ANTIALIAS_ON);
        g2d.setRenderingHint(RenderingHints.KEY_ANTIALIASING,RenderingHints.VALUE_ANTIALIAS_ON);
		synchronized (this) {
			if (model==null) {
				return;
			}
			int width = model.getWidth();//.length;
			int height = model.getHeight();//board[0].length;
			int cellWidth = (getWidth()-1)/width;
			int cellHeight = (getHeight()-1)/height;
			int cellSize = cellWidth<cellHeight?cellWidth:cellHeight;
			int leftPad = 0;
			int topPad = 0;
			
			g.setColor(Color.BLACK);
			for(int x = 0; x <= width; x++) {
				g.drawLine(leftPad+x*cellSize, topPad, leftPad+x*cellSize, topPad+height*cellSize);
			}
			for(int y = 0; y <= height; y++) {
				g.drawLine(leftPad, topPad+y*cellSize, leftPad+width*cellSize, topPad+y*cellSize);
			}
			
			int leftCenterPad = leftPad+cellSize/6;
			int topCenterPad = topPad+cellSize/6;
			int r = cellSize*2/3;
			for (int x=0; x<width; x++) {
				for (int y=0; y<height; y++) {
					int color = model.getValue(x,y);//board[x][y];
					if (color>=0) {
						g.setColor(COLOR_POOL[color]);
						g.fillOval(leftCenterPad+x*cellSize,topCenterPad+y*cellSize,r,r);
						g.setColor(Color.BLACK);
						g.drawOval(leftCenterPad+x*cellSize,topCenterPad+y*cellSize,r,r);
					}else{
                        g.setColor(Color.GRAY);

                        g.drawString("["+x+","+y+"]",leftCenterPad+x*cellSize,topCenterPad+y*cellSize+cellSize/2);
                    }
				}
			}
		}
	}

	void updatePlayground() {
		//copy(pg);

		SwingUtilities.invokeLater(new Runnable() {
			public void run() {
				repaint();
			}
		});
	}
}
