package cz.cvut.agents.rph.reversi;

import cz.cvut.agents.rph.core.GameResult;

/**
 * Encapsulates results of a reversi game for both players.
 * Author: Ondrej Vanek
 * Date: 10/3/12
 * Time: 2:49 PM
 */
public class ReversiGameResult extends GameResult {

    private PlayerResult player1Result;
    private PlayerResult player2Result;

    public ReversiGameResult(PlayerResult player1Result, PlayerResult player2Result) {
        this.player1Result = player1Result;
        this.player2Result = player2Result;
    }

    public PlayerResult getPlayer1Result() {
        return player1Result;
    }

    public PlayerResult getPlayer2Result() {
        return player2Result;
    }

    public void show() {
        System.out.println("Player 1 result: ");
        System.out.println(player1Result);
        System.out.println("Player2 result: ");
        System.out.println(player2Result);
    }

    @Override
    public String toString() {
        final StringBuilder sb = new StringBuilder();
        sb.append("P1 ").append(player1Result);//.append("\n");
        sb.append(" | P2 ").append(player2Result);
        return sb.toString();
    }

    public boolean proceededWithoutErrorsAndTimeouts() {
        return (player1Result.getResultType() != ReversiScoreEvaluator.ResultType.ERROR
                && player1Result.getResultType() != ReversiScoreEvaluator.ResultType.TIMEOUT
                && player2Result.getResultType() != ReversiScoreEvaluator.ResultType.ERROR
                && player2Result.getResultType() != ReversiScoreEvaluator.ResultType.TIMEOUT);
    }

    public  String getCSVlinePerPlayer(){
        return player1Result.getCSVLine()+ "\n"+player2Result.getCSVLine();
    }

    public String getCSVHeader() {
        return player2Result.getCSVHeader();
    }
}
