package cz.cvut.agents.rph.reversi;

import java.io.Serializable;

/**
 * A class representing one reversi move.
 * Author: Ondrej Vanek
 * Date: 10/3/12
 * Time: 10:52 AM
 */
public class ReversiMove implements Serializable {

    /**
     * The answer to life, the universe and everything.
     */
    private static final long serialVersionUID = 42L;

    /**
     * X coordinate of the stone.
     */
    private int x;

    /**
     * Y coordinate of the stone.
     */
    private int y;

    private long timeInMilis = 0;

    /**
     * Creates a move which can be processed by the {@link ReversiController}.
     * @param x X coordinate of the stone.
     * @param y Y coordinate of the stone.
     */
    public ReversiMove(int x, int y){
        this.x = x;
        this.y = y;
    }

    /**
     * Gets the duration of the move in miliseconds.
     * @return  duration of the move in miliseconds.
     */
    long getTimeInMilis() {
        return timeInMilis;
    }

    /**
     * Sets the duration of the move in miliseconds.
     * @param timeInMilis duration of the move in miliseconds.
     */
    void setTimeInMilis(long timeInMilis) {
        this.timeInMilis = timeInMilis;
    }

    /**
     * Gets X coordinate of the stone.
     * @return X coordinate of the stone.
     */
    public int getX() {
        return x;
    }

    /**
     * Get Y coordinate of the stone.
     * @return Y coordinate of the stone.
     */
    public int getY() {
        return y;
    }

    @Override
    public String toString() {
        final StringBuilder sb = new StringBuilder();
        sb.append("Move");
        sb.append("{x=").append(x);
        sb.append(", y=").append(y);
        sb.append('}');
        return sb.toString();
    }
}
