package cz.cvut.agents.rph.reversi;

import java.rmi.RemoteException;

/**
 * General template for a reversi player. Extend this class and place your player
 * in the {@link cz.cvut.agents.rph.reversi.players} package.
 * Author: Ondrej Vanek
 * Date: 10/3/12
 * Time: 10:09 AM
 */
public abstract class ReversiPlayer implements ReversiPlayerInterface {

    /**
     * Width of the board.
     */
    protected int width;

    /**
     *  Height of the board.
     */
    protected int height;

    /**
     * Color of my stones.
     */
    protected int myColor;

    /**
     * Color of opponent's stones.
     */
    protected int opponentColor;

    public ReversiPlayer(){
        //Explicit constructor for the ability to create the player by non-parametric constructor.
        //All necessary data are handled in the init method.
    }

    /**
     * {@inheritDoc}
     * The method is overridden if you do not want to implement anything.
     * @param boardWidth WIDTH of the board
     * @param boardHeight HEIGHT of the board
     * @param myColor color of the player
     * @param opponentColor color of the opponent
     */
    public void init(int boardWidth, int boardHeight, int myColor, int opponentColor){

    }

    @Override
    public final void systemInit(int boardWidth, int boardHeight, int myColor, int opponentColor) throws RemoteException {
        this.width = boardWidth;
        this.height = boardHeight;
        this.myColor = myColor;
        this.opponentColor = opponentColor;
        init(boardWidth,boardHeight,myColor,opponentColor);
    }

    /**
     * Gets the color of the player.
     * @return color of the player.
     */
    public final int getMyColor() {
        return myColor;
    }

    /**
     * Gets the color of the opponent player.
     * @return  color of the opponent player.
     */
    public final int getOpponentColor() {
        return opponentColor;
    }


    /**
     * Gets the color of the empty square.
     * @return  color of the empty square.
     */
    public final int getEmptySquareColor(){
        return ReversiModel.EMPTY_SQUARE;
    }

    /**
     * {@inheritDoc}
     * @return
     */
    public abstract String getName();

    /**
     * {@inheritDoc}
     * @param board - contains actual map of the board represented by two-dimensional matrix.
     *              Each sqaure contains number of -1 representing empty square and number 0 and 1 representing players' stones.
     * @return
     */
    public abstract ReversiMove makeNextMove(int[][] board);

    @Override
    public String toString() {
        final StringBuilder sb = new StringBuilder();
        sb.append("Player ").append(getName()).append(" ");
        sb.append("{color =").append(myColor);
        sb.append('}');
        return sb.toString();
    }

    /**
     * This method is used only in the tournament.
     */
    final public void endGame() {
        //Do nothing - used in the tournament.
    }
}

