package cz.cvut.agents.rph.reversi;

/**
 * Evaluates result of a reversi game.
 * Author: Ondrej Vanek
 * Date: 10/3/12
 * Time: 2:52 PM
 */
public class ReversiScoreEvaluator {

    public enum ResultType {WON,LOST,TIE,TIMEOUT,ERROR};

    /**
     * Creates result from the data available.
     * @param player1Stones number of stones of player 1.
     * @param player2Stones number of stones of player 2.
     * @param player1Timeout timeout of player 1?
     * @param player2timeout timeout of player 2?
     * @param player1Error error of player 1?
     * @param player2Error error of player 2?
     * @param player1name name of player 1.
     * @param player2name name of player 2.
     * @return result of the game.
     */
    public static ReversiGameResult evaluate(int player1Stones, int player2Stones, boolean player1Timeout, boolean player2timeout, boolean player1Error, boolean player2Error, String player1name, String player2name){

        ResultType p1resultType;
        ResultType p2resultType;

        int player1Score = 0;
        int player2Score = 0;
        if(player2timeout || player2Error){
            player1Score = 2;
            p1resultType = ResultType.WON;
            if(player2Error){
                p2resultType = ResultType.ERROR;
            }   else{
                p2resultType = ResultType.TIMEOUT;
            }
        }else if(player1Timeout ||player1Error){
            player2Score=2;
            if(player1Error){
                p1resultType = ResultType.ERROR;
            }   else{
                p1resultType = ResultType.TIMEOUT;
            }
            p2resultType = ResultType.WON;
        }else if(player1Stones>player2Stones){
            player1Score=2;
            p1resultType = ResultType.WON;
            p2resultType = ResultType.LOST;
        }else if(player2Stones>player1Stones){
            player2Score=2;
            p1resultType = ResultType.LOST;
            p2resultType = ResultType.WON;
        }else{
            player1Score=1;
            player2Score=1;
            p1resultType = ResultType.TIE;
            p2resultType = ResultType.TIE;
        }
        PlayerResult player1Result = new PlayerResult(player1name,player1Score,player1Stones,p1resultType);
        PlayerResult player2Result = new PlayerResult(player2name,player2Score,player2Stones,p2resultType);

        ReversiGameResult gameResult = new ReversiGameResult(player1Result,player2Result);
        return gameResult;
    }


}
