package cz.cvut.agents.rph.reversi;

import cz.cvut.agents.rph.core.GameModel;
import cz.cvut.agents.rph.core.GameView;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Dictionary;
import java.util.Hashtable;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JSlider;
import javax.swing.SwingUtilities;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

/**
 * Specific view for the Reversi game.
 * Author: Ondrej Vanek
 * Date: 10/3/12
 * Time: 10:04 AM
 */
public class ReversiView extends GameView implements ActionListener, ChangeListener, DelayChanger {

    /**
     * Reversi game model.
     */
    private ReversiModel model;

    /**
     * Viewpanel holding all the visualization
     */
    private JPanel viewPanel;

    /**
     * Panel depicting the playground.
     */
    private PlaygroundPanel playgroundPanel;

    public ReversiView(final ReversiModel model){
        this.model = model;

        //Let the AWT Thread do it.
        SwingUtilities.invokeLater(new Runnable() {
            public void run() {
                playgroundPanel = new PlaygroundPanel(model);
                constructViewPanel();
                update();
            }
        });
    }

    /**
     * author Premysl Volf
     */
    private void constructViewPanel() {
        viewPanel = new JPanel();
        viewPanel.setLayout(new GridBagLayout());
        viewPanel.add(playgroundPanel,
                new GridBagConstraints(0, 0, 1, 2, 1.0, 1.0, GridBagConstraints.NORTHWEST, GridBagConstraints.BOTH, new Insets(5, 5, 5, 5), 0, 0));
        viewPanel.add(infoPanel,
                new GridBagConstraints(1, 0, 1, 1, 0.1, 0.0, GridBagConstraints.NORTH, GridBagConstraints.HORIZONTAL, new Insets(10, 10, 10, 10), 0, 0));
        viewPanel.add(buttonPanel,
                new GridBagConstraints(1, 1, 1, 1, 0.1, 1.0, GridBagConstraints.SOUTHEAST, GridBagConstraints.NONE, new Insets(10, 10, 10, 10), 0, 0));

        infoPanel.setLayout(new GridBagLayout());
        jLabel3.setText("Current stones : ");
        jLabel6.setText("Max time : ");

        p1NameLabel.setForeground(PlaygroundPanel.COLOR_POOL[ReversiModel.PLAYER1_COLOR]);
        p1NameLabel.setFont(p1NameLabel.getFont().deriveFont(Font.BOLD));
        p1NameLabel.setText("Player 1 - " + model.getPlayer1Name());
        p2NameLabel.setForeground(PlaygroundPanel.COLOR_POOL[ReversiModel.PLAYER2_COLOR]);
        p2NameLabel.setFont(p2NameLabel.getFont().deriveFont(Font.BOLD));
        p2NameLabel.setText("Player 2 - " + model.getPlayer2Name());

        infoPanel.add(statusLabel,
                new GridBagConstraints(1, 0, 3, 1, 0.0, 0.0, GridBagConstraints.NORTHWEST, GridBagConstraints.BOTH, new Insets(0, 0, 0, 0), 0, 0));
        infoPanel.add(jLabel3,
                new GridBagConstraints(0, 3, 1, 1, 0.0, 0.0, GridBagConstraints.NORTHWEST, GridBagConstraints.BOTH, new Insets(10, 0, 0, 10), 0, 0));
        infoPanel.add(jLabel6,
                new GridBagConstraints(0, 6, 1, 1, 0.0, 0.0, GridBagConstraints.NORTHWEST, GridBagConstraints.BOTH, new Insets(10, 0, 0, 10), 0, 0));

        infoPanel.add(p1NameLabel,
                new GridBagConstraints(1, 2, 1, 1, 0.0, 0.0, GridBagConstraints.NORTHWEST, GridBagConstraints.BOTH, new Insets(30, 0, 0, 30), 0, 0));
        infoPanel.add(p1StonesLabel,
                new GridBagConstraints(1, 3, 1, 1, 0.0, 0.0, GridBagConstraints.NORTHWEST, GridBagConstraints.BOTH, new Insets(10, 0, 0, 30), 0, 0));
        infoPanel.add(p1TotalStonesLabel,
                new GridBagConstraints(1, 5, 1, 1, 0.0, 0.0, GridBagConstraints.NORTHWEST, GridBagConstraints.BOTH, new Insets(10, 0, 0, 30), 0, 0));
        infoPanel.add(p1MaxTimeLabel,
                new GridBagConstraints(1, 6, 1, 1, 0.0, 0.0, GridBagConstraints.NORTHWEST, GridBagConstraints.BOTH, new Insets(10, 0, 0, 30), 0, 0));

        infoPanel.add(p2NameLabel,
                new GridBagConstraints(2, 2, 1, 1, 0.0, 0.0, GridBagConstraints.NORTHWEST, GridBagConstraints.BOTH, new Insets(30, 0, 0, 10), 0, 0));
        infoPanel.add(p2StonesLabel,
                new GridBagConstraints(2, 3, 1, 1, 0.0, 0.0, GridBagConstraints.NORTHWEST, GridBagConstraints.BOTH, new Insets(10, 0, 0, 10), 0, 0));
        infoPanel.add(p2TotalStonesLabel,
                new GridBagConstraints(2, 5, 1, 1, 0.0, 0.0, GridBagConstraints.NORTHWEST, GridBagConstraints.BOTH, new Insets(10, 0, 0, 10), 0, 0));
        infoPanel.add(p2MaxTimeLabel,
                new GridBagConstraints(2, 6, 1, 1, 0.0, 0.0, GridBagConstraints.NORTHWEST, GridBagConstraints.BOTH, new Insets(10, 0, 0, 10), 0, 0));

        playPauseButton.setPreferredSize(new Dimension(150,60));
        playPauseButton.setMaximumSize(new Dimension(150,60));
        playPauseButton.setMinimumSize(new Dimension(150,60));
        playPauseButton.setFont(playPauseButton.getFont().deriveFont(Font.BOLD,20));
        playPauseButton.setText("Pause");
        playPauseButton.setToolTipText("Pauses the game");
        playPauseButton.setActionCommand(ReversiController.PLAY_PAUSE_ACTION_COMMAND);
        playPauseButton.addActionListener(this);

        createSlider();

        JPanel sliderPanel = new JPanel(new GridBagLayout());
        final GridBagConstraints layoutConf = new GridBagConstraints();
        { // each component fills the whole horizontal space and is laid at x=0
            layoutConf.fill = GridBagConstraints.HORIZONTAL;
            layoutConf.weightx = 1;
            layoutConf.gridx = 0;
        }
        JLabel delayLabel = new JLabel("Game speed");
        delayLabel.setPreferredSize(new Dimension(100,20));
        sliderPanel.add(delayLabel, layoutConf);
        sliderPanel.add(slider, layoutConf);
        slider.setPreferredSize(new Dimension(200, 50));
        sliderPanel.setPreferredSize(new Dimension(250,70));

        buttonPanel.setLayout(new BoxLayout(buttonPanel, BoxLayout.LINE_AXIS));
        buttonPanel.add(sliderPanel);
        buttonPanel.add(Box.createRigidArea(new Dimension(50,0)));
        buttonPanel.add(playPauseButton);
    }

    private void createSlider() {
        slider.addChangeListener(this);
       // slider.setMajorTickSpacing(1);
        slider.setPaintLabels(true);
        slider.setSnapToTicks(false);

        //Create the label table
        Dictionary<Integer,JLabel> labelTable = new Hashtable<Integer,JLabel>();
        labelTable.put( new Integer( SLIDER_MIN ), new JLabel("Fast (4ms/move)") );
        labelTable.put( new Integer( SLIDER_MAX-SLIDER_MAX/50 ), new JLabel("Slow (4s/move)") );
        slider.setLabelTable(labelTable);
    }

    @Override
    public void update() {
        switch(model.getGameStatus()){
            case PLAYING: updateButton(GameModel.GameStatus.PLAYING); break;
            case PAUSED: updateButton(GameModel.GameStatus.PAUSED); break;
            case FINISHED: updateButton(GameModel.GameStatus.FINISHED); showEndGame(); break;
            default: throw new IllegalArgumentException("Unknown game status: " + model.getGameStatus());
        }

        //Update GUI in the AWT Thread.
        SwingUtilities.invokeLater(new Runnable() {
            public void run() {
                p1StonesLabel.setText(model.getPlayer1Stones()+"");
                p2StonesLabel.setText(model.getPlayer2Stones()+"");

                playgroundPanel.updatePlayground();

                p1MaxTimeLabel.setText(model.getPlayer1MaxTime()  + " [ms]");
                p2MaxTimeLabel.setText(model.getPlayer2MaxTime()  + " [ms]");

                p1NameLabel.setText("Player 1 - " + model.getPlayer1Name());
                p2NameLabel.setText("Player 2 - " + model.getPlayer2Name());
            }
        });
    }

    /**
     * Shows the end of the game dialog with some info about the result.
     */
    private void showEndGame() {
        final String timeoutPlayer = model.getTimeoutPlayerName();
        final String errorPlayer = model.getErrorPlayerName();
        final ReversiGameResult reversiGameResult = model.getGameResult();
        String message = "GAME OVER.\n";
        if(timeoutPlayer.length()>0){
            message+="\nTimeout player: "+timeoutPlayer;
        }
        if(errorPlayer.length()>0){
            message+="\nError player: "+ errorPlayer;
        }
        final String finalMessage = message+"\n\n"+reversiGameResult;

        SwingUtilities.invokeLater(new Runnable() {
            public void run() {
                if(timeoutPlayer.length()>0 || errorPlayer.length()>0){
                    JOptionPane.showMessageDialog(viewPanel,finalMessage,"Game Ended by timeout or error!",JOptionPane.WARNING_MESSAGE);
                }
                else{
                    JOptionPane.showMessageDialog(viewPanel,
                            finalMessage,"Game ended normally.",JOptionPane.INFORMATION_MESSAGE);
                }
            }
        });

    }

    /**
     * Update the button status.
     * @param playing are we playing or what?
     */
    private void updateButton(final ReversiModel.GameStatus playing) {
        SwingUtilities.invokeLater(new Runnable() {
            public void run() {
                if (playing.equals(GameModel.GameStatus.PLAYING)) {
                    playPauseButton.setActionCommand(ReversiController.PLAY_PAUSE_ACTION_COMMAND);
                    playPauseButton.setText(GameModel.GameStatus.getButtonText(playing));
                } else if (playing.equals(GameModel.GameStatus.PAUSED)) {
                    playPauseButton.setActionCommand(ReversiController.PLAY_PAUSE_ACTION_COMMAND);
                    playPauseButton.setText(GameModel.GameStatus.getButtonText(playing));
                } else {
                    playPauseButton.setActionCommand(ReversiController.RESTART_ACTION_COMMAND);
                    playPauseButton.setText(GameModel.GameStatus.getButtonText(playing));
                }
            }
        });
    }

    private JPanel infoPanel = new JPanel();
    private JPanel buttonPanel = new JPanel();
    private JLabel statusLabel = new JLabel();

    private JLabel jLabel3 = new JLabel();
    private JLabel jLabel6 = new JLabel();

    private JLabel p1NameLabel = new JLabel();
    private JLabel p1StonesLabel = new JLabel();
    private JLabel p1TotalStonesLabel = new JLabel();
    private JLabel p1MaxTimeLabel = new JLabel();

    private JLabel p2NameLabel = new JLabel();
    private JLabel p2StonesLabel = new JLabel();
    private JLabel p2TotalStonesLabel = new JLabel();
    private JLabel p2MaxTimeLabel = new JLabel();

    private JButton playPauseButton = new JButton();

    public static final int SLIDER_MIN = 200;
    public static final int SLIDER_MAX = 1200;
    public static final int SLIDER_VALUE = 500;
    private JSlider slider = new JSlider(JSlider.HORIZONTAL, SLIDER_MIN, SLIDER_MAX, SLIDER_VALUE);

    public Component getPanel() {
        return viewPanel;
    }

    /**
     * {@inheritDoc}
     * @param e
     */
    public void actionPerformed(ActionEvent e) {
        notifyListeners(e);
    }

    public void stateChanged(ChangeEvent e) {
        JSlider source = (JSlider) e.getSource();

       // if (!source.getValueIsAdjusting()) {
            source.getValue();
            notifyListeners(new ActionEvent(this,0,ReversiController.AFTER_MOVE_DELAY_CHANGED));
       // }
    }

    public int getDelayInMiliseconds() {
        final int POWER_BASE = 2;

        int value = slider.getValue()/100;

        //the value is decadic logarithm.
        value = (int) Math.pow(POWER_BASE,value);
        return value;
    }
}