package cz.cvut.agents.rph.reversi.players;


import cz.cvut.agents.rph.reversi.ReversiMove;
import cz.cvut.agents.rph.reversi.ReversiPlayer;

/**
 * Dummy player is an example of implementation of the player. It plays random and after each move it checks
 * if the move is valid ("real" player should always check validity of the turn by itself).
 */
public final class DummyPlayer extends ReversiPlayer {
	/**
	 * Constant defining maximum number of tried moves. If it is exceeded, player systematically tries all moves.
	 */
	private final int RANDOM_MOVES_NUMBER = 1000;


    @Override
	public String getName() {

		return "Dummy Player";
	}

    @Override
    public ReversiMove makeNextMove(int[][] board) {
        // try predefined number of random moves
        for (int i=0; i<RANDOM_MOVES_NUMBER; i++) {
            // generate random position of stone
            int randomX = (int)(Math.random()*width);
            int randomY = (int)(Math.random()*height);
            // try to apply this move
            if (isCorrectMove(board,this,new ReversiMove(randomX,randomY))) {
                // finish if successful
                return new ReversiMove(randomX,randomY);
            }
            // continue if not successful
            // real player HAS TO CHECK VALIDITY BY ITSELF
        }
        // if random moves fail, try avery position on the playground
        for (int x=0; x<width; x++) {
            for (int y=0; y<height; y++) {
                if (isCorrectMove(board, this, new ReversiMove(x, y))) {
                    // finish if successful
                    return new ReversiMove(x,y);
                }else{
                    System.out.println("Incorrect: "+x+","+y);
                }
            }
        }
        System.out.println("I don't know, beeee :(");
        return new ReversiMove(-1,-1);
    }

    public boolean isCorrectMove(int[][] board, ReversiPlayer player, ReversiMove move) {
        if(move==null) return  false;
        int x = move.getX();
        int y = move.getY();
        int color = player.getMyColor();
        if (x<0 || x>=width || y<0 || y>=height) {
            return false;
        }
        if (board[x][y]>=0) {
            return false;
        }
        boolean direction = false;
        direction |= isCorrectMoveInDirection(board,color, 1, 0, x, y);
        direction |= isCorrectMoveInDirection(board, color, 1, 1, x, y);
        direction |= isCorrectMoveInDirection(board, color, 0, 1, x, y);
        direction |= isCorrectMoveInDirection(board, color, -1, 1, x, y);
        direction |= isCorrectMoveInDirection(board, color, -1, 0, x, y);
        direction |= isCorrectMoveInDirection(board, color, -1, -1, x, y);
        direction |= isCorrectMoveInDirection(board, color, 0, -1, x, y);
        direction |= isCorrectMoveInDirection(board, color, 1, -1, x, y);

        return direction;
    }

    private boolean isCorrectMoveInDirection(int[][] board, int player, int dirX, int dirY, int x, int y) {
        boolean neighbourOpponent = false;
        boolean endingMyStone = false;
        while (!endingMyStone) {
            x += dirX;
            y += dirY;
            if (x<0 || x>=width || y<0 || y>=height) {
                return false;
            }
            int color = board[x][y];
            if (color<0) {
                return false;
            }
            if (color!=player) {
                neighbourOpponent = true;
            }
            else {
                endingMyStone = true;
            }
        }
        if (neighbourOpponent && endingMyStone) {
            return true;
        }
        return false;
    }



}
